import{d as k,bQ as V,f as t,aS as B,aT as D,b,o as a,q as T,n as C,k as n,l as e,bR as N,aY as W,g as $,W as j,al as U,e as q,bu as A,bS as E,c as i,i as o,j as F,u as J,bT as M,a as p,bU as O,bV as P,bD as Q,a6 as Y,bk as Z}from"./index-BBlvj0M_.js";import{s as z,b as G}from"./index-0yGGjo7r.js";import{u as H}from"./usePageTitle-lBdMRcrQ.js";const K={class:"flow-stats"},L={class:"flow-stats__cards"},X=k({__name:"FlowStats",props:{flowId:{}},setup(w){const f=w,r={range:{type:"span",seconds:-G},tags:[]},{flowId:u}=V(f),c=t(()=>({flowId:u.value,range:r.range}));B(D,{interval:z(30)});const l=t(()=>b.map("FlowStatsFilter",c.value,"FlowRunsFilter")),d=t(()=>b.map("FlowStatsFilter",c.value,"TaskRunsFilter"));return(_,m)=>(a(),T("div",K,[C("div",L,[n(e(N),{filter:l.value},null,8,["filter"]),n(e(W),{filter:d.value},null,8,["filter"])])]))}}),oe=k({__name:"Flow",setup(w){const f=$(),r=j("flowId"),u=t(()=>[r.value]),c=J(),l=U("tab","Runs"),d=["Runs","Deployments","Details"],_={interval:3e5},m=q(f.flows.getFlow,[r.value],_),s=t(()=>m.response),{filter:R}=A({flows:{id:u}}),{filter:y}=E({flows:{id:u}});function g(){c.push(Z.flows())}const x=t(()=>s.value?`Flow: ${s.value.name}`:"Flow");return H(x),(ee,v)=>{const I=F("p-tabs"),S=F("p-layout-default");return a(),i(S,{class:"flow"},{header:o(()=>[s.value?(a(),i(e(M),{key:0,flow:s.value,onDelete:g},null,8,["flow"])):p("",!0)]),default:o(()=>[s.value?(a(),i(X,{key:0,"flow-id":s.value.id},null,8,["flow-id"])):p("",!0),n(I,{selected:e(l),"onUpdate:selected":v[0]||(v[0]=h=>Y(l)?l.value=h:null),tabs:d},{details:o(()=>[s.value?(a(),i(e(O),{key:0,flow:s.value},null,8,["flow"])):p("",!0)]),deployments:o(()=>[n(e(P),{filter:e(y),prefix:"deployments"},null,8,["filter"])]),runs:o(()=>[n(e(Q),{filter:e(R),selectable:"",prefix:"runs"},null,8,["filter"])]),_:1},8,["selected"])]),_:1})}}});export{oe as default};
//# sourceMappingURL=Flow-Cc9kc3aw.js.map
