import{d as b,g as f,ae as y,f as o,cj as m,e as _,c as d,i as s,j as g,o as v,k as t,l,ck as C,cl as B}from"./index-BBlvj0M_.js";import{u as T}from"./usePageTitle-lBdMRcrQ.js";const w=b({__name:"BlocksCatalog",setup(x){const c=f(),a=y(null),p=o(()=>a.value?[a.value]:[]),{filter:n}=m({blockSchemas:{blockCapabilities:p}}),i=_(c.blockTypes.getBlockTypes,[n]),u=o(()=>i.response??[]);return T("Blocks Catalog"),(h,e)=>{const r=g("p-layout-default");return v(),d(r,{class:"blocks-catalog"},{header:s(()=>[t(l(C))]),default:s(()=>[t(l(B),{capability:a.value,"onUpdate:capability":e[0]||(e[0]=k=>a.value=k),"block-types":u.value},null,8,["capability","block-types"])]),_:1})}}});export{w as default};
//# sourceMappingURL=BlocksCatalog-FEmz88zM.js.map
