import{d as e,c as o,l as a,o as t,dC as r}from"./index-BBlvj0M_.js";import{u as s}from"./usePageTitle-lBdMRcrQ.js";const m=e({__name:"404",setup(n){return s("Page Not Found"),(c,p)=>(t(),o(a(r)))}});export{m as default};
//# sourceMappingURL=404-D9J7kR3b.js.map
