import{d as h,g as w,W as x,f as l,al as D,bt as R,e as S,c,i as n,j as u,u as I,aL as T,o as a,l as t,cP as j,a as r,cQ as m,k as B,cR as N,a6 as P,bk as V}from"./index-BBlvj0M_.js";import{u as W}from"./usePageTitle-lBdMRcrQ.js";const Q=h({__name:"ConcurrencyLimit",setup(Z){const p=w(),y=x("concurrencyLimitId"),v=I(),d=l(()=>[{label:"Details",hidden:T.xl},{label:"Active Task Runs"}]),s=D("tab","Details"),{tabs:_}=R(d,s),b={interval:3e5},L=S(p.concurrencyLimits.getConcurrencyLimit,[y.value],b),e=l(()=>L.response);function f(){v.push(V.concurrencyLimits())}const k=l(()=>e.value?`Concurrency Limit: ${e.value.tag}`:"Concurrency Limit");return W(k),(A,i)=>{const C=u("p-tabs"),g=u("p-layout-well");return a(),c(g,{class:"concurrencyLimit"},{header:n(()=>[e.value?(a(),c(t(j),{key:0,"concurrency-limit":e.value,onDelete:f},null,8,["concurrency-limit"])):r("",!0)]),well:n(()=>[e.value?(a(),c(t(m),{key:0,alternate:"","concurrency-limit":e.value},null,8,["concurrency-limit"])):r("",!0)]),default:n(()=>[B(C,{selected:t(s),"onUpdate:selected":i[0]||(i[0]=o=>P(s)?s.value=o:null),tabs:t(_)},{details:n(()=>[e.value?(a(),c(t(m),{key:0,"concurrency-limit":e.value},null,8,["concurrency-limit"])):r("",!0)]),"active-task-runs":n(()=>{var o;return[(o=e.value)!=null&&o.activeSlots?(a(),c(t(N),{key:0,"active-slots":e.value.activeSlots},null,8,["active-slots"])):r("",!0)]}),_:1},8,["selected","tabs"])]),_:1})}}});export{Q as default};
//# sourceMappingURL=ConcurrencyLimit-DRDsWy4d.js.map
