import{d as c,g as _,W as u,J as k,c as m,i as l,j as d,o as i,k as r,l as e,d8 as w,d9 as f}from"./index-BBlvj0M_.js";import{u as P}from"./usePageTitle-lBdMRcrQ.js";const x=c({__name:"WorkPoolEdit",async setup(y){let o,a;const s=_(),n=u("workPoolName"),t=([o,a]=k(()=>s.workPools.getWorkPoolByName(n.value)),o=await o,a(),o);return P("Create Work Pool"),(C,W)=>{const p=d("p-layout-default");return i(),m(p,null,{header:l(()=>[r(e(w),{"work-pool":e(t)},null,8,["work-pool"])]),default:l(()=>[r(e(f),{"work-pool":e(t)},null,8,["work-pool"])]),_:1})}}});export{x as default};
//# sourceMappingURL=WorkPoolEdit-qhFuo-E-.js.map
