import{d as k,g as c,W as t,J as w,c as _,i as r,j as d,o as i,k as n,l as o,df as f,dg as P}from"./index-BBlvj0M_.js";import{u as Q}from"./usePageTitle-lBdMRcrQ.js";const q=k({__name:"WorkPoolQueueEdit",async setup(N){let e,u;const s=c(),a=t("workPoolName"),l=t("workPoolQueueName"),p=([e,u]=w(()=>s.workPoolQueues.getWorkPoolQueueByName(a.value,l.value)),e=await e,u(),e);return Q("Edit Work Pool Queue"),(g,y)=>{const m=d("p-layout-default");return i(),_(m,null,{header:r(()=>[n(o(f),{"work-pool-name":o(a),"work-pool-queue-name":o(l)},null,8,["work-pool-name","work-pool-queue-name"])]),default:r(()=>[n(o(P),{"work-pool-name":o(a),"work-pool-queue":o(p)},null,8,["work-pool-name","work-pool-queue"])]),_:1})}}});export{q as default};
//# sourceMappingURL=WorkPoolQueueEdit-RZaaiGhc.js.map
