import{d as g,g as k,e as v,f as s,c as n,i,j as r,o as a,k as l,l as o,cx as x,C as y,q as N,cy as C,cz as B,z as D,a as T}from"./index-BBlvj0M_.js";import{u as V}from"./usePageTitle-lBdMRcrQ.js";const P=g({__name:"Notifications",setup(b){const u=k(),t=v(u.notifications.getNotifications),c=s(()=>t.response??[]),f=s(()=>t.executed&&c.value.length===0),p=s(()=>t.executed);return V("Notifications"),(w,e)=>{const d=r("p-message"),m=r("p-layout-default");return a(),n(m,{class:"notifications"},{header:i(()=>[l(o(x))]),default:i(()=>[l(d,{info:""},{default:i(()=>e[2]||(e[2]=[y(" Notifications are deprecated and will be migrated in the future. Please use Automations. ")])),_:1}),p.value?(a(),N(D,{key:0},[f.value?(a(),n(o(C),{key:0})):(a(),n(o(B),{key:1,notifications:c.value,onDelete:e[0]||(e[0]=_=>o(t).refresh()),onUpdate:e[1]||(e[1]=_=>o(t).refresh())},null,8,["notifications"]))],64)):T("",!0)]),_:1})}}});export{P as default};
//# sourceMappingURL=Notifications-DQsYhmkT.js.map
