import{d,ae as m,g as _,f as o,G as v,c as r,i as c,j as b,e as k,o as t,k as g,l as n,cX as h,q as y,cY as C,cZ as x,z as V,a as B}from"./index-BBlvj0M_.js";import{u as I}from"./usePageTitle-lBdMRcrQ.js";const w=d({__name:"Variables",setup(N){const l=m(),a=()=>{var e;s.value.refresh(),(e=l.value)==null||e.refreshSubscriptions()},u=_(),s=o(()=>k(u.variables.getVariables)),i=o(()=>{var e;return s.value.executed&&((e=s.value.response)==null?void 0:e.length)===0}),p=o(()=>s.value.executed);return I(v.info.variables),(e,S)=>{const f=b("p-layout-default");return t(),r(f,{class:"variables"},{header:c(()=>[g(n(h),{onCreate:a})]),default:c(()=>[p.value?(t(),y(V,{key:0},[i.value?(t(),r(n(C),{key:0,onCreate:a})):(t(),r(n(x),{key:1,ref_key:"table",ref:l,onDelete:a,onUpdate:a},null,512))],64)):B("",!0)]),_:1})}}});export{w as default};
//# sourceMappingURL=Variables-C8dEQReD.js.map
