import{f as a,bw as c,l as n}from"./index-BBlvj0M_.js";function f(...t){const r=[...t,"Prefect Server"],o=a(()=>r.map(e=>n(e)).filter(e=>e!==null).join(" • "));c(()=>document.title=o.value)}export{f as u};
//# sourceMappingURL=usePageTitle-lBdMRcrQ.js.map
