import{d as m,g as i,e as _,f as o,c as s,i as n,j as w,o as t,k as C,l as a,bN as k,q as b,bO as v,bP as y,z as g,a as h}from"./index-BBlvj0M_.js";import{u as x}from"./usePageTitle-lBdMRcrQ.js";const P=m({__name:"Flows",setup(F){const l=i(),c={interval:3e4},e=_(l.flows.getFlowsCount,[{}],c),r=o(()=>e.response??0),u=o(()=>e.executed&&r.value===0),p=o(()=>e.executed),d=()=>{e.refresh()};return x("Flows"),(B,N)=>{const f=w("p-layout-default");return t(),s(f,{class:"flows"},{header:n(()=>[C(a(k))]),default:n(()=>[p.value?(t(),b(g,{key:0},[u.value?(t(),s(a(v),{key:0})):(t(),s(a(y),{key:1,selectable:"",onDelete:d}))],64)):h("",!0)]),_:1})}}});export{P as default};
//# sourceMappingURL=Flows-veMajAfY.js.map
