import{d as u,s,w as c,c as l,r as m,a as p,F as i,h as f,b as h,o as v,B as d}from"./index-BBlvj0M_.js";const r=a=>{const{value:e,isCustom:o}=i();if(f(a.query)&&o.value)try{const t=h.map("SavedSearchFilter",e.value,"LocationQuery");return{...a,query:t}}catch(t){console.error(t)}return!0},C=u({beforeRouteEnter:r,beforeRouteUpdate:r,__name:"RunsPageWithDefaultFilter",props:{component:{type:Function}},setup(a){const e=a,o=s(null);function t(n){return d(n)}return c(e.component,()=>{t(e.component)?e.component().then(n=>{o.value=n.default}):o.value=e.component},{immediate:!0}),(n,y)=>o.value!==null?(v(),l(m(o.value),{key:0})):p("",!0)}});export{C as default};
//# sourceMappingURL=RunsPageWithDefaultFilter-DPAcZL_D-DrEprHeu.js.map
