import{d as m,g as i,e as k,f as t,c as a,i as n,j as _,o,k as f,l as s,d3 as P,q as g,d4 as x,d5 as y,z as h,a as v}from"./index-BBlvj0M_.js";import{u as w}from"./usePageTitle-lBdMRcrQ.js";const U=m({__name:"WorkPools",setup(B){const r=i(),l={interval:3e4},e=k(r.workPools.getWorkPools,[{}],l),c=t(()=>e.response??[]),p=t(()=>e.executed&&c.value.length==0),u=t(()=>e.executed);return w("Work Pools"),(C,W)=>{const d=_("p-layout-default");return o(),a(d,{class:"work-pools"},{header:n(()=>[f(s(P))]),default:n(()=>[u.value?(o(),g(h,{key:0},[p.value?(o(),a(s(x),{key:0})):(o(),a(s(y),{key:1,onUpdate:s(e).refresh},null,8,["onUpdate"]))],64)):v("",!0)]),_:1})}}});export{U as default};
//# sourceMappingURL=WorkPools-nEw2DZmo.js.map
