import{d as m,g as y,W as _,e as k,f as s,c as t,i as o,j as v,G as n,o as e,l as c,aO as d,a as r,aK as g,q as C,aP as K,z as h}from"./index-BBlvj0M_.js";import{u as x}from"./usePageTitle-lBdMRcrQ.js";const W=m({__name:"ArtifactKey",setup(B){const l=y(),u=_("artifactKey"),i=k(l.artifacts.getArtifactCollection,[u]),a=s(()=>i.response),f=s(()=>a.value?`${n.info.artifact}: ${a.value.key}`:n.info.artifact);return x(f),(A,N)=>{const p=v("p-layout-default");return e(),t(p,{class:"artifact"},{header:o(()=>[a.value?(e(),t(c(d),{key:0,artifact:a.value},null,8,["artifact"])):r("",!0)]),default:o(()=>[a.value?(e(),t(c(g),{key:0,artifact:a.value},null,8,["artifact"])):r("",!0),a.value?(e(),C(h,{key:1},[a.value.key?(e(),t(c(K),{key:0,"artifact-key":a.value.key},null,8,["artifact-key"])):r("",!0)],64)):r("",!0)]),_:1})}}});export{W as default};
//# sourceMappingURL=ArtifactKey-B7EN33PD.js.map
