import{d as o,c as r,i as a,j as c,o as n,k as e,l as t,aQ as _,aR as f}from"./index-BBlvj0M_.js";import{u as l}from"./usePageTitle-lBdMRcrQ.js";const h=o({__name:"Artifacts",setup(p){return l("Artifacts"),(u,i)=>{const s=c("p-layout-default");return n(),r(s,{class:"artifacts"},{header:a(()=>[e(t(_))]),default:a(()=>[e(t(f))]),_:1})}}});export{h as default};
//# sourceMappingURL=Artifacts-5AXbeEpJ.js.map
