import{d as v,f as y,o as i,q as u,k as e,i as t,z as x,x as C,l as n,dz as B,t as f,J as w,c as A,j as l,K as P,dA as V,dB as p,a6 as j}from"./index-BBlvj0M_.js";import{u as z}from"./usePageTitle-lBdMRcrQ.js";import{u as K}from"./usePrefectApi-Boduij1X.js";import"./api-D3LJrWSC.js";import"./mapper-DWfgs_2K.js";const M={class:"settings-block"},T=v({__name:"SettingsCodeBlock",props:{engineSettings:{}},setup(d){const s=d,o=y(()=>Object.entries(s.engineSettings));return(g,c)=>(i(),u("div",M,[e(n(B),{multiline:""},{default:t(()=>[(i(!0),u(x,null,C(o.value,(a,r)=>(i(),u("div",{key:r,class:"settings-block--code-line"},f(a[0])+": "+f(a[1]),1))),128))]),_:1})]))}}),L=v({__name:"Settings",async setup(d){let s,o;const g=[{text:"Settings"}],c=K(),[a,r]=([s,o]=w(()=>Promise.all([c.admin.getSettings(),c.admin.getVersion()])),s=await s,o(),s);return z("Settings"),(q,m)=>{const S=l("p-key-value"),b=l("p-theme-toggle"),_=l("p-label"),k=l("p-layout-default");return i(),A(k,{class:"settings"},{header:t(()=>[e(n(P),{crumbs:g},{actions:t(()=>[e(S,{class:"settings__version",label:"Version",value:n(r),alternate:""},null,8,["value"])]),_:1})]),default:t(()=>[e(_,{label:"Theme"},{default:t(()=>[e(b)]),_:1}),e(_,{label:"Color Mode",class:"settings__color-mode"},{default:t(()=>[e(n(V),{selected:n(p),"onUpdate:selected":m[0]||(m[0]=h=>j(p)?p.value=h:null)},null,8,["selected"])]),_:1}),e(_,{label:"Server Settings"},{default:t(()=>[e(T,{class:"settings__code-block","engine-settings":n(a)},null,8,["engine-settings"])]),_:1})]),_:1})}}});export{L as default};
//# sourceMappingURL=Settings-CmJWo04-.js.map
