import{d as B,g as D,W as R,e as S,f as p,ae as T,al as V,c,i as e,j as n,G as _,aH as $,o as l,l as t,aI as h,a as i,aJ as d,q as H,k as s,aK as L,aL as W,aM as v,C as j,t as q,aN as m,a6 as E}from"./index-BBlvj0M_.js";import{u as G}from"./usePageTitle-lBdMRcrQ.js";const J={key:0},X=B({__name:"Artifact",setup(K){const b=D(),k=R("artifactId"),w=S(b.artifacts.getArtifact,[k]),a=p(()=>w.response),r=T(!1),y=[{label:"Artifact"},{label:"Details"},{label:"Raw"}],u=V("tab","Artifact"),g=p(()=>a.value?`${_.info.artifact}: ${a.value.key??$(a.value.type)}`:_.info.artifact);return G(g),(M,o)=>{const C=n("p-divider"),x=n("p-button"),A=n("p-content"),I=n("p-tabs"),N=n("p-layout-well");return l(),c(N,{class:"artifact"},{header:e(()=>[a.value?(l(),c(t(h),{key:0,artifact:a.value},null,8,["artifact"])):i("",!0)]),well:e(()=>[a.value?(l(),c(t(d),{key:0,artifact:a.value,alternate:""},null,8,["artifact"])):i("",!0)]),default:e(()=>[a.value?(l(),H("section",J,[s(t(L),{artifact:a.value},null,8,["artifact"]),s(C),t(W).xl?(l(),c(A,{key:0},{default:e(()=>[s(t(v),{artifact:a.value},null,8,["artifact"]),s(x,{class:"artifact__raw-data-button",small:"",onClick:o[0]||(o[0]=f=>r.value=!r.value)},{default:e(()=>[j(q(r.value?"Hide":"Show")+" raw data ",1)]),_:1}),r.value?(l(),c(t(m),{key:0,artifact:a.value},null,8,["artifact"])):i("",!0)]),_:1})):(l(),c(I,{key:1,selected:t(u),"onUpdate:selected":o[1]||(o[1]=f=>E(u)?u.value=f:null),tabs:y},{artifact:e(()=>[s(t(v),{artifact:a.value},null,8,["artifact"])]),details:e(()=>[s(t(d),{artifact:a.value},null,8,["artifact"])]),raw:e(()=>[s(t(m),{artifact:a.value},null,8,["artifact"])]),_:1},8,["selected"]))])):i("",!0)]),_:1})}}});export{X as default};
//# sourceMappingURL=Artifact-C_rk3SH7.js.map
