import{d as k,g as i,W as y,f as o,bG as b,c as m,i as a,a as _,j as f,o as T,k as t,l as c,cm as d,cn as g}from"./index-BBlvj0M_.js";import{u as v}from"./usePageTitle-lBdMRcrQ.js";const x=k({__name:"BlocksCatalogView",setup(B){const s=i(),l=y("blockTypeSlug"),n=o(()=>l.value?[l.value]:null),u=b(s.blockTypes.getBlockTypeBySlug,n),e=o(()=>u.response),p=o(()=>e.value?`Block Type: ${e.value.name}`:null);return v(p),(C,S)=>{const r=f("p-layout-default");return e.value?(T(),m(r,{key:0,class:"blocks-catalog-view"},{header:a(()=>[t(c(d),{"block-type":e.value},null,8,["block-type"])]),default:a(()=>[t(c(g),{"block-type":e.value},null,8,["block-type"])]),_:1})):_("",!0)}}});export{x as default};
//# sourceMappingURL=BlocksCatalogView-D51j1jvX.js.map
