import{d as p,g as l,c as _,i as o,j as m,o as d,k as e,l as n,cA as C,cB as h,c8 as i,bk as c,P as k}from"./index-BBlvj0M_.js";import{u as g}from"./usePageTitle-lBdMRcrQ.js";const w=p({__name:"NotificationCreate",setup(y){const r=l();async function s(t){try{await r.notifications.createNotification(t),i.push(c.notifications())}catch(a){k("Error creating notification","error"),console.warn(a)}}function u(){i.push(c.notifications())}return g("Create Notification"),(t,a)=>{const f=m("p-layout-default");return d(),_(f,{class:"notification-create"},{header:o(()=>[e(n(C))]),default:o(()=>[e(n(h),{action:"Create",onSubmit:s,onCancel:u})]),_:1})}}});export{w as default};
//# sourceMappingURL=NotificationCreate-DWiebzhT.js.map
