import{d as i,g as b,W as d,f as o,bG as _,c as f,i as c,a as v,j as D,u as B,o as g,k as u,l as n,ct as h,cu as C,bk as V}from"./index-BBlvj0M_.js";import{u as w}from"./usePageTitle-lBdMRcrQ.js";const S=i({__name:"BlockView",setup(x){const s=b(),l=B(),t=d("blockDocumentId"),a=o(()=>t.value?[t.value]:null),r=_(s.blockDocuments.getBlockDocument,a),e=o(()=>r.response),m=()=>{l.push(V.blocks())},k=o(()=>e.value?`Block: ${e.value.name}`:"Block");return w(k),(y,A)=>{const p=D("p-layout-default");return e.value?(g(),f(p,{key:0,class:"block-view"},{header:c(()=>[u(n(h),{"block-document":e.value,onDelete:m},null,8,["block-document"])]),default:c(()=>[u(n(C),{"block-document":e.value},null,8,["block-document"])]),_:1})):v("",!0)}}});export{S as default};
//# sourceMappingURL=BlockView-BxsZM3vQ.js.map
