import{d,g as i,e as y,f as o,c as a,i as n,j as _,o as t,k as f,l as s,bW as g,q as k,bX as v,bV as D,z as b,a as h}from"./index-BBlvj0M_.js";import{u as x}from"./usePageTitle-lBdMRcrQ.js";const q=d({__name:"Deployments",setup(C){const l=i(),c={interval:3e4},e=y(l.deployments.getDeployments,[{}],c),p=o(()=>e.response??[]),r=o(()=>e.executed&&p.value.length===0),m=o(()=>e.executed);return x("Deployments"),(B,V)=>{const u=_("p-layout-default");return t(),a(u,{class:"deployments"},{header:n(()=>[f(s(g))]),default:n(()=>[m.value?(t(),k(b,{key:0},[r.value?(t(),a(s(v),{key:0})):(t(),a(s(D),{key:1,onDelete:s(e).refresh},null,8,["onDelete"]))],64)):h("",!0)]),_:1})}}});export{q as default};
//# sourceMappingURL=Deployments-CKHBBAIj.js.map
