import{d as f,g as x,V as b,W as c,f as r,aD as y,J as C,l as e,c as E,i as n,a as g,j as h,aE as k,o as D,k as s,aF as F,K as V,aG as w}from"./index-BBlvj0M_.js";import{u as I}from"./usePageTitle-lBdMRcrQ.js";const W=f({__name:"Event",async setup(B){let a,o;const l=x(),u=b(),v=c("eventDate"),_=c("eventId"),m=r(()=>k(v.value)),d=y({startDate:m,eventId:[_.value]}),t=([a,o]=C(()=>l.events.getFirstEvent(d.value)),a=await a,o(),a),p=r(()=>[{text:"Event Feed",to:u.events()},{text:t.eventLabel}]);return I(`Event: ${t.eventLabel}`),(G,K)=>{const i=h("p-layout-default");return e(t)?(D(),E(i,{key:0,class:"event"},{header:n(()=>[s(e(V),{crumbs:p.value},{actions:n(()=>[s(e(F),{event:e(t)},null,8,["event"])]),_:1},8,["crumbs"])]),default:n(()=>[s(e(w),{event:e(t)},null,8,["event"])]),_:1})):g("",!0)}}});export{W as default};
//# sourceMappingURL=Event-BhCY2DZD.js.map
