import{d as p,g as m,e as d,f as a,c as s,i as c,j as _,o as t,k,l as o,cg as f,q as i,ch as g,ci as y,z as B,a as h}from"./index-BBlvj0M_.js";import{u as x}from"./usePageTitle-lBdMRcrQ.js";const V=p({__name:"Blocks",setup(C){const n=m(),e=d(n.blockDocuments.getBlockDocumentsCount),l=a(()=>e.executed&&e.response==0),r=a(()=>e.executed);return x("Blocks"),(D,b)=>{const u=_("p-layout-default");return t(),s(u,{class:"blocks"},{header:c(()=>[k(o(f))]),default:c(()=>[r.value?(t(),i(B,{key:0},[l.value?(t(),s(o(g),{key:0})):(t(),s(o(y),{key:1,onDelete:o(e).refresh},null,8,["onDelete"]))],64)):h("",!0)]),_:1})}}});export{V as default};
//# sourceMappingURL=Blocks-rxHVk9YB.js.map
